package flare.vis
{
	import flare.data.DataSet;
	import flare.vis.axis.Axes;
	import flare.vis.axis.CartesianAxes;
	import flare.vis.controls.TooltipControl;
	import flare.vis.data.Data;
	import flare.vis.data.NodeMode;
	import flare.vis.data.render.EdgeRenderer;
	import flare.vis.data.render.IRenderer;
	import flare.vis.data.skins.EdgeSkin;
	import flare.vis.events.TooltipEvent;
	
	import flash.display.Sprite;
	import flash.events.Event;
	
	import mx.core.IDataRenderer;
	import mx.core.IToolTip;
	import mx.core.UIComponent;
	import mx.events.FlexEvent;
	
	public class FlareVisualization extends UIComponent implements IDataRenderer
	{
		// ========================================
		// Config properties
		// ========================================	
		
		/**
		 * Currently keeping renderer in here, for the sake of doing stuff to the nodes if we need to.
		 * Hopefully we can just remove the concept of nodeRenderer entirely though.
		
		public var nodeRenderer:IRenderer;
		 */
		
		/**
		 * Skin to apply to nodes within this visualisation. They must  
		 */		
		public var nodeSkin:Class;
		
		/**
		 * Specify here to change the renderer used for edges, usually leave it alone as it handles the 
		 * drawing of the lines
		 */		
		public var edgeRenderer:IRenderer = EdgeRenderer.instance;
		
		
		/**
		 * Edges will still need to render using the renderer method, as it is really
		 * a drawing process rather than a skin...  but you can still apply a skin to display
		 * other things on an edge if you wish. This defaults to an empty skin.
		 */		
		public var edgeSkin:Class = EdgeSkin;	
		
		
		/**
		 * If you want the toolTip arg on the nodes to be set to
		 * the contents of a field of the data, specify the fieldname here 
		 */
		public var tooltipField:String;
		
		public var leftPadding:Number = 20;
		public var topPadding:Number = 20;
		
		
		// ========================================
		// Public properties
		// ========================================	
		
		/**
		 * Flare Visualization instance.
		 */
		public function get visualization():Visualization {
			return _visualization;
		}
		
		/** 
		 * 	The visualization operators used by this visualization. This
		 *  should be an array of IOperator instances. 
		 */
		public function set operators(a:Array):void {
			if(_visualization){
				_visualization.operators.list = a;
				_visualization.update();
			}
			_operators = a;
		}
		
		private var _operators:Array;
		
		/** 
		 * 	The interactive controls used by this visualization. This
		 *  should be an array of IControl instances. 
		 */
		public function set controls(a:Array):void {
			if(_visualization){
				_visualization.controls.list = a;
				_visualization.update();
			}
			_controls = a;
		}
		
		private var _controls:Array;
		
		/** 
		 * Returns the axes for the backing visualization instance. 
		 */
		public function get axes():Axes 			{ return _visualization.axes; }
		
		/** 
		 * Returns the CartesianAxes for the backing visualization instance. 
		 */
		public function get xyAxes():CartesianAxes 	{ return _visualization.xyAxes; }

		
		
		/** 
		 *  Legacy support and wrapper to function set data(val) method 
		 */
		public function set dataSet(d:*):void 		{	this.data = d;		}

		[Bindable("dataChange")]
		/** 
		 *  Sets the data visualized by this instance. The input value can be
		 *  an array of data objects, a Data instance, or a DataSet instance.
		 *  Any existing data will be removed and new NodeMode instances will
		 *  be created for each object in the input arrary. 
		 */
		public function get data():Object {
			return _data;
		}
		
		public function set data( value:Object ):void {
			if ( _data != value ) {				
				_data = value;
				
				var d:Data = _data as Data;
				dataChanged = true;
				invalidateProperties();
				commitProperties();							
				dispatchEvent( new FlexEvent( FlexEvent.DATA_CHANGE ) );
			}
		}
		
		// ========================================
		// Constructor
		// ========================================	
		
		/**
		 * Constructor.
		 */
		public function FlareVisualization(data:Data=null)
		{
			super();
			
			if(!data){
				data = new Data();
			}			
			_data = data;
			
			_visualization = new Visualization( _data as Data );
			_visualization.removeEventListener( Event.RENDER, _visualization.setHitArea );
			_visualization.x = leftPadding;
			_visualization.y = topPadding;	
			
			
			//Set the controls and operators again now that we 
			if(_controls){
				controls = _controls;
			}
			if(_operators){
				operators = _operators;
			}
		}
		
		
		
		
		
		// ========================================
		// Public Methods
		// ========================================
		
		
		/**
		 * This takes an array of objects and returns the Flare Data object
		 * it builds from it. You must override this in your extended class
		 * as the form of your data is usually quite unique. Example code 
		 * left commented out here for reference. You should also have this function
		 * set the data object.
		 * @param arr Input array
		 * @return Flare Data object
		 * 
		 */		
		public function buildData(arr:Array):Data{
			/*var d:Data = new Data(false,nodeSkin,edgeSkin,'description');					
			var root:NodeMode = d.createRootNode();			
			root.data.type = 'root';
			root.data.description = 'The entire World!';
			
			
			//Keyed lookup for easy edge addition in step 2
			var nodeLookup:Object = {};
			var row:Object;
			var ns:NodeMode;				
			
			//Step 1: Add all rows of data;			
			for each(row in arr){
				ns = d.addNode(row);				
				nodeLookup[row.id] = ns; 
			}
			
			//Step 2: Add edges			
			for each(ns in d.nodes){
				if(ns.data.type == 'host'){
					d.addEdgeFor(root,ns);
				}
				if(ns.data.hasOwnProperty('host')){
					d.addEdgeFor(nodeLookup[ns.data.host],ns);
				}
				else if(ns.data.hasOwnProperty('dir')){
					d.addEdgeFor(nodeLookup[ns.data.dir],ns);
				}
			}					
			this.data = d;			
			return d;*/
			
			return null;
		}
		
		// ========================================
		// Protected methods
		// ========================================	
				
		
		/**
		 * @inheritDoc
		 */
		override protected function createChildren():void
		{
			super.createChildren();
			
			addChild( _visualization );
			
			visualizationMask = new Sprite();
			addChild( visualizationMask );
			
			_visualization.mask = visualizationMask;
		}
		
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if ( dataChanged )
			{
				dataChanged = false;
				
				if ( data == null )			_visualization.data = null;
				else if ( data is Data) 	_visualization.data = data as Data;
				else if ( data is Array )	_visualization.data = Data.fromArray( data as Array);
				else if ( data is DataSet )	_visualization.data = Data.fromDataSet( data as DataSet);
				else						throw new Error( "Unrecognized data set type: " + data );
				
				_visualization.operators.setup();
				_visualization.update();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function updateDisplayList( unscaledWidth:Number, unscaledHeight:Number ):void
		{
			super.updateDisplayList( unscaledWidth, unscaledHeight );
			
			// draw 'hitbox'
			
			graphics.clear();
			
			graphics.beginFill( 0xffffff, 0.0 );
			graphics.drawRect( 0, 0, unscaledWidth, unscaledHeight );
			graphics.endFill();
			
			// update visualization mask
			
			visualizationMask.graphics.clear();
			
			visualizationMask.graphics.beginFill( 0xff0000 );
			visualizationMask.graphics.drawRect( 0, 0, unscaledWidth, unscaledHeight );
			visualizationMask.graphics.endFill();
			
			visualizationMask.cacheAsBitmap = true;
			_visualization.update();
			trace('VIS update Display List');
		}
		
		// ========================================
		// Protected properties
		// ========================================	
		
		/**
		 * Backing variable for <code>visualization</code> property.
		 */
		protected var _visualization:Visualization;
		
		/**
		 * Backing variable for <code>data</code> property.
		 */
		protected var _data:Object;
		
		/**
		 * Indicates the <code>data</code> property was invalidated.
		 */
		protected var dataChanged:Boolean;
		
		/**
		 * Mask for Flare Visualization.
		 */
		protected var visualizationMask:Sprite;		
	}
}
